/* ccgo: view_game.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "view_game.hh"

using namespace go;

void ViewGame::on_realize()
{
	Gtk::DrawingArea::on_realize();
	win = get_window();
	style = get_style();
}

void ViewGame::on_unrealize()
{
	Gtk::DrawingArea::on_unrealize();
}

bool ViewGame::on_configure_event(GdkEventConfigure * e)
{
	win_w = e->width;
	win_h = e->height;
	int z = get_board_size();
	if (z) {
		cell_w = win_w / z - 1;
		cell_h = win_h / z - 1;
	} else {
		cell_w = 0;
		cell_h = 0;
	}
	return true;
}

bool ViewGame::on_expose_event(GdkEventExpose * e)
{
	// std::cerr << "ViewGame exposed" << std::endl;
	if (e->count > 0) return true;
	int z = get_board_size();
	for (int x = 0; x < z; ++ x) for (int y = 0; y < z; ++ y) draw_loc(Loc(x, y));
	return true;
}

void ViewGame::draw_loc_at(const Loc & l, int cx, int cy)
{
	int z = get_board_size();
	int x = l.get_x();
	int y = l.get_y();
	win->draw_rectangle(style->get_bg_gc(get_state()), true, cx - cell_w / 2, cy - cell_h / 2, cell_w, cell_h);
	switch (get_site(l)) {
	case STATE_WHITE:
		win->draw_arc(style->get_white_gc(), true, cx - cell_w / 2, cy - cell_h / 2, cell_w, cell_h, 0, 64 * 360);
		win->draw_arc(style->get_black_gc(), false, cx - cell_w / 2, cy - cell_h / 2, cell_w, cell_h, 0, 64 * 360);
		break;
	case STATE_BLACK:
		win->draw_arc(style->get_black_gc(), true, cx - cell_w / 2, cy - cell_h / 2, cell_w, cell_h, 0, 64 * 360);
		win->draw_arc(style->get_white_gc(), false, cx - cell_w / 2, cy - cell_h / 2, cell_w, cell_h, 0, 64 * 360);
		break;
	case STATE_EMPTY:
		win->draw_arc(style->get_bg_gc(get_state()), false, cx - cell_w / 2, cy - cell_h / 2, cell_w, cell_h, 0, 64 * 360);
		win->draw_line(style->get_fg_gc(get_state()), (x == 0 ? cx : cx - cell_w / 2 - 1), cy, (x == z - 1 ? cx : cx + cell_w / 2 + 1), cy);
		win->draw_line(style->get_fg_gc(get_state()), cx, (y == z - 1 ? cy : cy - cell_h / 2 - 1), cx, (y == 0 ? cy : cy + cell_h / 2 + 1));
		if ((z == 19 && (x == 3 || x == 9 || x == 15) && (y == 3 || y == 9 || y == 15)) || (z == 13 && (x == 3 || x == 6 || x == 9) && (y == 3 || y == 6 || y == 9)) || (z == 9 && (x == 4 && y == 4 || (x == 2 || x == 6) && (y == 2 || y == 6)))) win->draw_arc(style->get_fg_gc(get_state()), true, cx - cell_w / 6, cy - cell_h / 6, cell_w / 3, cell_h / 3, 0, 64 * 360); // star positions
	}
	if (l == get_ko()) {
		win->draw_line(style->get_black_gc(), cx + cell_w / 4, cy - cell_h / 4, cx - cell_w / 4, cy + cell_h / 4);
		win->draw_line(style->get_black_gc(), cx - cell_w / 4, cy - cell_h / 4, cx + cell_w / 4, cy + cell_h / 4);
	}
	if (l == get_last_move()) {
		switch (get_site(l)) {
		case STATE_WHITE:
			win->draw_arc(style->get_black_gc(), false, cx - cell_w / 4, cy - cell_h / 4, cell_w / 2, cell_h / 2, 0, 64 * 360);
			break;
		case STATE_BLACK:
			win->draw_arc(style->get_white_gc(), false, cx - cell_w / 4, cy - cell_h / 4, cell_w / 2, cell_h / 2, 0, 64 * 360);
			break;
		case STATE_EMPTY:
			win->draw_arc(style->get_fg_gc(get_state()), false, cx - cell_w / 4, cy - cell_h / 4, cell_w / 2, cell_h / 2, 0, 64 * 360);
			break;
		}
	}
}

void ViewGame::draw_loc(const Loc & l)
{
	if (! is_realized() || l == Loc()) return;
	int cx;
	int cy;
	trans_loc(l, cx, cy);
	draw_loc_at(l, cx, cy);
}

void ViewGame::trans_loc(const Loc & l, int & cx, int & cy)
{
	int z = get_board_size();
	cx = (2 * l.get_x() + 1) * win_w / z / 2;
	cy = (2 * (z - 1 - l.get_y()) + 1) * win_h / z / 2;
}

ViewGame::ViewGame() :
	Game()
{
}

ViewGame::~ViewGame()
{
	// std::cerr << "deleting ViewGame" << std::endl;
}

void ViewGame::set_board_size(unsigned z)
{
	Game::set_board_size(z);
	if (is_realized() && z) {
		cell_w = win_w / z - 1;
		cell_h = win_h / z - 1;
	} else {
		cell_w = 0;
		cell_h = 0;
	}
}

void ViewGame::set_ko(const Loc & l)
{
	Loc old_ko = get_ko();
	Game::set_ko(l);
	draw_loc(get_ko());
	draw_loc(old_ko);
}

void ViewGame::set_last_move(const Loc & l)
{
	Loc old_last_move = get_last_move();
	Game::set_last_move(l);
	draw_loc(get_last_move());
	draw_loc(old_last_move);
}

void ViewGame::set_site(const Loc & l, State s)
{
	Game::set_site(l, s);
	draw_loc(l);
}
